"""Create folders in FTP.

Use the /app/ftp_tree.yaml to get tree structure.

Attributes
----------
    BASE_DIR (TYPE): Base dir for .geru files

"""
import os

import arrow
import click
import docker
from docker.errors import ImageNotFound
from loguru import logger

from megalus.main import Megalus

client = docker.from_env()


def get_services(meg, service, service_list, ignore_list, tree, compose_data):
    if service in service_list or service in ignore_list:
        return

    service_list.append(service) if compose_data.get('build', None) else ignore_list.append(
        service)
    if compose_data.get('depends_on', None):
        tree = compose_data['depends_on']

    if tree:
        for key in tree:
            key_data = meg.find_service(key)
            get_services(meg, key, service_list, ignore_list, tree, key_data['compose_data'])
    return service_list, ignore_list


@click.command()
@click.argument('services', nargs=-1)
@click.pass_obj
def check(meg: Megalus, services):
    """Run main code."""

    def get_docker_image(compose_data):
        if compose_data.get('image', None) and compose_data.get('build', None):
            try:
                return client.images.get(compose_data['image'])
            except ImageNotFound:
                return None

    def has_old_image(ctx, service):

        def get_date_from_file(file):
            date = arrow.get(os.path.getmtime(os.path.join(data['working_dir'], file))).to('local')
            logger.debug("Last update for file {} is {}".format(file, date))
            return date

        data = ctx.find_service(service)
        image = get_docker_image(data['compose_data'])
        if not image:
            return False
        else:
            image_date_created = arrow.get(image.attrs['Created']).to('local')
            global_files_to_watch = meg.config_data['project'].get('check_for_build', {}).get('files', [])
            project_files_to_watch = meg.config_data['services'].get(service, {}).get('check_for_build', {}).get('files', [])
            all_files = global_files_to_watch + project_files_to_watch
            list_dates = [
                get_date_from_file(file)
                for file in all_files
                if os.path.isfile(os.path.join(data['working_dir'], file))  # FIXME: Acertar o working_dir + build.context para achar o path dos arquivos
            ]
            if list_dates and image_date_created < max(list_dates):
                return True
            return False


    service_list = []
    ignore_list = []
    for service in services:
        logger.info("Checking {}...".format(service))
        service_data = meg.find_service(service)
        service_list, ignore_list = get_services(
            meg, service, service_list, ignore_list, [],
            service_data['compose_data']
        )

    services_without_images = [
        service
        for service in service_list
        if get_docker_image(meg.find_service(service))
    ]

    if services_without_images:
        logger.warning("Services without images: {}".format(", ".join(services_without_images)))

    services_with_old_images = [
        service
        for service in service_list
        if service not in services_without_images and has_old_image(meg, service)
    ]

    if services_with_old_images:
        logger.warning("Services with old images: {}".format(", ".join(services_with_old_images)))



import os
import sys

import yaml
from buzio import console
from loguru import logger

from megalus.utils import get_path


class Megalus:

    def __init__(self, config_file):
        self.service = None
        self._config_file = config_file
        self.compose_data_list = []
        self._data = {}  # type: Dict[str, Any]
        self.all_services = []

    def _convert_lists(self, data, key):
        """Convert list to dict inside yaml data.

        Works only for Key=Value lists.

        Example:
            environment:
                - DEBUG=false
            ports:
                - "8090:8080"

        Result:
            environment: {"DEBUG": "false"}
            ports: ['8090:8080']

        """
        if isinstance(data[key], list) and "=" in data[key][0]:
            data[key] = {obj.split("=")[0]: obj.split("=")[-1] for obj in data[key]}
        if isinstance(data[key], dict):
            for k in data[key]:
                self._convert_lists(data[key], k)

    def _load_data_from_override(self, source, target, key):
        """Append override data in self.compose.

        Example Compose::
        ---------------
        core:
            build:
                context: ../core
            image: core
            networks:
                - backend
            environment:
                - DEBUG=false
            ports:
             - "8080:80"

        Example override::
        ----------------
        core:
            build:
                dockerfile: Docker_dev
            depends_on:
                - api
            command: bash -c "python manage.py runserver 0.0.0.0"
            environment:
                DEBUG: "True"
            ports:
                - "9000:80"

        Final Result::
        ------------
        core:
            build:
                context: ../core
                dockerfile: Docker_dev
            depends_on:
                - api
            image: core
            command: bash -c "python manage.py runserver 0.0.0.0"
            environment:
                DEBUG: "True"
            networks:
                - backend
            ports:
             - "8080:80"
             - "9000:80"

        """
        if target.get(key, None):
            if isinstance(source[key], dict):
                for k in source[key]:
                    self._load_data_from_override(
                        source=source[key],
                        target=target[key],
                        key=k
                    )
            else:
                if isinstance(target[key], list) and isinstance(source[key], list):
                    target[key] += source[key]
                else:
                    target[key] = source[key]
        else:
            if isinstance(target, list) and isinstance(source[key], list):
                target[key] += source[key]
            else:
                target[key] = source[key]

    def get_all_compose_data(self):
        for compose_subgroup_name in self.config_data['compose_files']:
            compose_paths = self.get_compose_files_for_group(compose_subgroup_name)
            compose_data = self.get_compose_data_for_group(compose_paths)
            for service in compose_data['services']:
                self.all_services.append(
                    {
                        'index': "{}_{}".format(service, compose_subgroup_name),
                        'name': service,
                        'subgroup': compose_subgroup_name,
                        'working_dir': os.path.dirname(compose_paths[0]),
                        'compose_data': compose_data['services'][service],
                        'all_compose_data': compose_data
                    }
                )

    def find_service(self, service_informed):

        exact_matches = [
            data
            for data in self.all_services
            if data['name'] == service_informed
        ]
        if len(exact_matches) == 1:
            self.service = exact_matches[0]['name']
            return exact_matches[0]

        eligible_services = [
            eligible_service
            for eligible_service in self.all_services
            if service_informed in eligible_service['name']
        ]
        if not eligible_services:
            logger.error("Service not found")
            sys.exit(1)
        elif len(eligible_services) == 1:
            self.service = eligible_services[0]['name']
            return eligible_services[0]
        else:
            choice_list = [
                data['index']
                for data in eligible_services
            ]
            service_name = console.choose(choice_list, 'Please select the service')
        data = [
            data
            for data in eligible_services
            if service_name == data['index']
        ][0]
        self.service = data['name']
        return data

    def get_compose_data_for_group(self, compose_paths) -> dict:
        """Read docker compose files data.

        :return: None
        """
        compose_data_list = []
        for compose_file in compose_paths:
            with open(compose_file, 'r') as file:
                compose_data = yaml.load(file.read())
                for key in compose_data:  # type: ignore
                    self._convert_lists(compose_data, key)
                compose_data_list.append(compose_data)
        reversed_list = list(reversed(compose_data_list))
        self._data = reversed_list[-1]
        for index, override in enumerate(reversed_list):
            self.override = override
            if index + 1 == len(reversed_list):
                break
            for key in self.override:
                self._load_data_from_override(self.override, self._data, key)
        return self._data

    def _resolve_path(self, compose):
        base_compose_path = os.path.dirname(compose)
        if "." in base_compose_path:
            base_compose_path = self.config_data['project']['working_dir']
        return get_path(compose, base_path=base_compose_path)

    def get_compose_files_for_group(self, group):
        return [
            self._resolve_path(compose)
            for compose in self.config_data['compose_files'][group]
        ]

    @property
    def working_dir(self):
        return self._resolve_path(self.config_data['project']['working_dir'])

    @property
    def config_data(self):
        with open(self._config_file) as file:
            config_data = yaml.load(file.read())
        return config_data

    def get_config_from_service(self, service, key):
        return self.config_data.get('services', {}).get(service, {}).get('config', {}).get(key, None)

    @property
    def run_before(self):
        return self.config_data['project'].get('run_before', "")

    @property
    def run_after(self):
        return self.config_data['project'].get('run_after', "")

    def run_command(self, command):
        env = self.get_environment()
        logger.debug("Running command: {}".format(command))
        command_to_run = "{run_before}{command}{run_after}".format(
            run_before="{} && ".format(self.run_before) if self.run_before else "",
            command=command,
            run_after=" && {}".format(self.run_after) if self.run_after else ""
        )
        ret = console.run(command_to_run)
        if not ret:
            sys.exit(1)
        return ret

    def get_environment(self):
        command_env = os.environ.copy()
        command_env["PATH"] = "/usr/sbin:/sbin:" + command_env["PATH"]
        for env in self.config_data['project'].get('environment'):
            command_env[env] = os.environ.get(env, self.config_data['project']['environment'].get(env, ""))
        for env in self.config_data['project'].get(self.service, {}).get('environment', []):
            command_env[env] = os.environ.get(env, self.config_data['project']['environment'].get(env, ""))
        return command_env



import datetime
import hashlib
import json
import os
import re
import shutil
import sys
from pathlib import Path

import docker
from buzio import console
from loguru import logger

client = docker.from_env()


def find_service(compose_file, service_informed):
    compose_services = compose_file['services']

    if service_informed in compose_services:
        service_name = service_informed
    else:
        eligible_services = [
            eligible_service
            for eligible_service in compose_services
            if service_informed in eligible_service
        ]
        if not eligible_services:
            logger.error("Service not found")
            sys.exit(1)
        elif len(eligible_services) == 1:
            service_name = eligible_services[0]
        else:
            service_name = console.choose(eligible_services, 'Please select the service')
    return service_name, compose_file['services'][service_name]


def find_containers(service):
    eligible_containers = [
        container
        for container in client.containers.list()
        if service in container.name
    ]
    return eligible_containers


def backup_folder(folder_path):
    folders = list(Path(folder_path).parts)
    folder_hash = hashlib.md5(datetime.datetime.now().isoformat().encode("utf-8")).hexdigest()
    last_folder = "{}_{}".format(folders[-1], folder_hash[:10])
    folders.remove(folders[-1])
    folders.append(last_folder)
    backup_path = os.path.join(*folders)
    logger.warning('Directory already exists. Moving folder to {}'.format(backup_path))
    shutil.move(folder_path, backup_path)


def run_command(meg, command):
    logger.debug("Running command: {}".format(command))
    command_to_run = "{run_before}{command}{run_after}".format(
        run_before="{} && ".format(meg.run_before) if meg.run_before else "",
        command=command,
        run_after=" && {}".format(meg.run_after) if meg.run_after else ""
    )
    ret = console.run(command_to_run)
    if not ret:
        sys.exit(1)


def get_path(path: str, base_path: str) -> str:
    """Return real path from string.

    Converts environment variables to path
    Converts relative path to full path
    """

    def _convert_env_to_path(env_in_path):
        s = re.search(r"\${(\w+)}", env_in_path)
        if not s:
            s = re.search(r"(\$\w+)", env_in_path)
        if s:
            env = s.group(1).replace("$", "")
            name = os.environ.get(env)
            if not name:
                raise ValueError("Can't find value for {}".format(env))
            path_list = [
                part if "$" not in part else name
                for part in env_in_path.split("/")
            ]
            path = os.path.join(*path_list)
        else:
            raise ValueError("Cant find path for {}".format(env_in_path))
        return path

    if "$" in base_path:
        base_path = _convert_env_to_path(base_path)
    if "$" in path:
        path = _convert_env_to_path(path)
    if path.startswith("."):
        list_path = os.path.join(base_path, path)
        path = os.path.abspath(list_path)
    return path


def update_service_status(service, key, value):
    status_file = os.path.join(str(Path.home()), '.megalus', 'service_status.json')
    with open(status_file, "w+") as file:
        data = file.read()
        service_status = json.loads(file.read()) if data else {}

        service_data = service_status.get(service, {})
        if not service_data:
            service_data = {service: {key: value}}
        else:
            service_data.update({key: value})
        service_status.update(service_data)

        file.write(json.dumps(service_status, indent=4))


def get_service_status(service):
    status_file = os.path.join(str(Path.home()), '.megalus', 'service_status.json')
    with open(status_file, "w+") as file:
        data = file.read()
        service_status = json.loads(file.read()) if data else {}
    return service_status.get(service, {})



import datetime
import os
import sys
from pathlib import Path

from loguru import logger

from megalus.utils import get_path

__version__ = "2.0.0a6"

BASE_LOG_PATH = os.path.join(str(Path.home()), '.megalus', 'logs')

if not os.path.exists(BASE_LOG_PATH):
    os.makedirs(BASE_LOG_PATH)

now = datetime.datetime.now().isoformat()
LOGFILE = os.path.join(BASE_LOG_PATH, '{}.log'.format(now))

config = {
    "handlers": [
        {"sink": sys.stdout},
        {"sink": LOGFILE, "retention": "7 days"}
    ],
}
logger.configure(**config)



"""
commands.
"""
import sys

import click
from buzio import console
from loguru import logger

from megalus.main import Megalus
from megalus.utils import run_command


@click.command()
@click.argument('commands', nargs=-1, required=True)
@click.pass_obj
def run(meg: Megalus, commands: list):
    for command in commands:
        line_to_run = meg.config_data['project'].get('scripts', {}).get(command, None)
        if not line_to_run:
            logger.warning('Command "{}" not found in configuration file.'.format(command))
        else:
            run_command(meg, line_to_run)




import click

from megalus.compose.commands import run_compose_command


def stop_all(meg):
    compose_set = set([
        data['working_dir']
        for data in meg.all_services
    ])
    for compose_dir in list(compose_set):
        meg.run_command("cd {} && docker-compose stop".format(compose_dir))


@click.command()
@click.argument('services', nargs=-1)
@click.pass_obj
def stop(meg, services):
    if not services:
        stop_all(meg)
    for service in services:
        service_data = meg.find_service(service)
        run_compose_command(meg, "stop", service_data)



"""
clone.
"""
import os
from pathlib import Path
from typing import List

import click
from loguru import logger

from megalus import LOGFILE
from megalus.main import Megalus
from megalus.utils import backup_folder


def get_clone_path(service_data: dict) -> Path:
    path = service_data['compose_data'].get('build', {}).get('context', None)
    clone_path = Path(os.path.join(service_data['working_dir'], path)).resolve()
    logger.debug("Clone path for service {} is: {}".format(service_data['name'], clone_path))
    return clone_path


@click.command()
@click.argument('services', nargs=-1, required=True)
@click.option('--force', is_flag=True)
@click.pass_obj
def clone(meg: Megalus, services: List, force: bool) -> None:
    for service in services:
        service_data = meg.find_service(service)

        clone_path = get_clone_path(service_data)
        if os.path.exists(clone_path):
            if not force:
                logger.warning("Ignoring path {} - already exists.".format(clone_path))
                continue
            else:
                backup_folder(clone_path)

        repository_url = meg.get_config_from_service(service, "repository")
        if repository_url:
            logger.info("Cloning {} in path {}".format(repository_url, clone_path))
            meg.run_command(
                "git clone --progress {} {} 2> {}".format(repository_url, clone_path, LOGFILE)
            )



"""
commands.
"""
import sys
from typing import Optional

import arrow
import click
from loguru import logger

from megalus.main import Megalus
from megalus.utils import update_service_status


def find_and_run_command(meg, service, action):
    if not service:
        config_command = meg.config_data['project'].get('commands').get(action, None)
    else:
        config_command = meg.config_data['services'].get(service, {}).get('commands', {}).get(action, {})
    if config_command:
        meg.run_command(config_command)
        update_service_status(service, "last_{}".format(action), arrow.utcnow().to('local').isoformat())
    else:
        logger.error("No command defined in configuration.")
        sys.exit(1)


@click.command()
@click.argument('service', required=False)
@click.pass_obj
def config(meg: Megalus, service: Optional[str]):
    find_and_run_command(meg, service, "config")


@click.command()
@click.argument('service', required=False)
@click.pass_obj
def install(meg: Megalus, service: Optional[str]):
    find_and_run_command(meg, service, "install")


@click.command()
@click.argument('service', required=False)
@click.pass_obj
def init(meg: Megalus, service: Optional[str]):
    find_and_run_command(meg, service, "init")


@click.command()
@click.argument('service', required=False)
@click.pass_obj
def reset(meg: Megalus, service: Optional[str]):
    find_and_run_command(meg, service, "reset")


@click.command()
@click.argument('service', required=False)
@click.pass_obj
def update(meg: Megalus, service: Optional[str]):
    find_and_run_command(meg, service, "update")



"""
BUILD
Usage: geru build [--no-cache]| [<service> ...]
This command will build the selected service (using docker-compose build).
Use the --no-cache option to force installing without cache
If you do not inform any service, the command will make the minimal install
**Do not start this until you have tested your docker install.**

This command will make the minimum install: ***core***, ***migrate***,
***awscli***, ***geru-ngrok***, ***ftp*** and ***monitor*** services.

If you have issues with docker cache during install, use the command
`geru forcebuild <service>` to make install.


##### Troubleshooting:

1. Check if the repositories are in their default branches (ie. stage)
2. Check if you have permission to read the repositories
3. Check if you git personal token has all "repo" operations marked
4. Check if your docker are working correctly typing `docker run busybox
  nslookup google.com`
5. Check you have at least 40Gb free disk space.
6. Check your internet connection. You can run again `geru build` (the
   build will resume the operation)
"""
from typing import List

import arrow as arrow
import click
from loguru import logger

from megalus import LOGFILE
from megalus.main import Megalus
from megalus.utils import update_service_status


def _build_services(meg, force, services) -> None:
    for service_to_find in services:
        logger.info('Looking for Service: {}'.format(service_to_find))
        service_data = meg.find_service(service_to_find)

        meg.run_command(
            'cd {} && docker-compose build --force-rm {}{} | pv -lft -D 2 >> {}'.format(
                service_data['working_dir'],
                " --no-cache " if force else "",
                service_data['name'],
                LOGFILE
            ))
        update_service_status(service_data['name'], "last_build", arrow.utcnow().to('local').isoformat())
        logger.success('Service {} builded.'.format(service_data['name']))



@click.command()
@click.argument('services', nargs=-1, required=True)
@click.option('--force', is_flag=True)
@click.pass_obj
def build(meg: Megalus, services: List, force: bool) -> None:
    _build_services(meg, force, services)


@click.command()
@click.argument('groups', nargs=-1, required=True)
@click.option('--force', is_flag=True)
@click.pass_obj
def buildgroup(meg: Megalus, groups: List, force: bool) -> None:
    all_groups = set(
        [
            meg.get_config_from_service(service, "group")
            for service in meg.config_data.get('services', {})
            if meg.get_config_from_service(service, "group") is not None
        ]
    )
    for group in groups:
        if group in all_groups:
            logger.info("Build group {}".format(group))
            group_services = [
                service
                for service in meg.config_data['services']
                if meg.get_config_from_service(service, "group") == group
            ]
            _build_services(meg, force, group_services)
        else:
            logger.warning("Group {} not found".format(group))



import click

from megalus.check.commands import check
from megalus.main import Megalus
from megalus.bash.commands import bash
from megalus.build.commands import build, buildgroup
from megalus.clone.commands import clone
from megalus.commands.commands import config, install, reset, update, init
from megalus.compose.commands import restart, scale, up
from megalus.stop.commands import stop
from megalus.down.commands import down
from megalus.logs.commands import logs
from megalus.run.commands import run


@click.group()
@click.option('--config_file', envvar='MEGALUS_PROJECT_CONFIG_FILE', required=True, type=click.Path())
@click.pass_context
def cli(ctx, config_file):
    meg = Megalus(config_file=config_file)
    meg.get_all_compose_data()
    ctx.obj = meg


cli.add_command(bash)
cli.add_command(build)
cli.add_command(buildgroup)
cli.add_command(clone)
cli.add_command(config)
cli.add_command(down)
cli.add_command(install)
cli.add_command(logs)
cli.add_command(reset)
cli.add_command(restart)
cli.add_command(run)
cli.add_command(scale)
cli.add_command(stop)
cli.add_command(up)
cli.add_command(update)
cli.add_command(check)
cli.add_command(init)

def start():
    cli()



"""
commands.
"""

import click


def run_compose_command(meg, action, service_data, options=""):
    meg.run_command(
        "cd {working_dir} && docker-compose {action}{options}{services}".format(
            working_dir=service_data['working_dir'],
            options=" {} ".format(options) if options else " ",
            action=action,
            services=service_data['name']
        )
    )


@click.command()
@click.argument('services', nargs=-1, required=True)
@click.pass_obj
def restart(meg, services):
    for service in services:
        service_data = meg.find_service(service)
        run_compose_command(meg, "restart", service_data)


@click.command()
@click.argument('service', required=True)
@click.argument('number', required=True, default=1, type=click.INT)
@click.pass_obj
def scale(meg, service, number):
    service_data = meg.find_service(service)
    options = "-d --scale {}={}".format(service_data['name'], number)
    run_compose_command(meg, "up", options=options, service_data=service_data)


@click.command()
@click.argument('services', nargs=-1, required=True)
@click.option('-d', is_flag=True)
@click.pass_obj
def up(meg, services, d):
    options = "-d" if d else ""
    for service in services:
        service_data = meg.find_service(service)
        run_compose_command(meg, "up", options=options, service_data=service_data)



import click
from loguru import logger

from megalus.main import Megalus


@click.command()
@click.option('--remove-all', is_flag=True)
@click.argument('subgroups', nargs=-1)
@click.pass_obj
def down(meg: Megalus, subgroups, remove_all):
    options = "--rmi all -v --remove-orphans" if remove_all else ""

    if not subgroups:
        subgroups = [
            subgroup
            for subgroup in meg.config_data['compose_files']
        ]

    all_working_dirs = set([
        data['working_dir']
        for subgroup in subgroups
        for data in meg.all_services
        if data['subgroup'] == subgroup
    ])
    logger.debug("All working directories: {}".format(", ".join(list(all_working_dirs))))
    for path in list(all_working_dirs):
        meg.run_command(
            "cd {working_dir} && docker-compose down {options}".format(
                working_dir=path,
                options=options
            )
        )



"""
commands.
"""
import re
import sys
from time import sleep

import arrow
import click
from buzio import console
from loguru import logger

from megalus.main import Megalus
from megalus.utils import find_containers, find_service


def show_log(name, line):
    config = {
        "handlers": [
            {
                "sink": sys.stderr,
                "format": "<d><lvl>{level: <8}</lvl> | <cyan>{extra[container]}</cyan> "
                          "| <blue>{extra[docker_timestamp]}</blue></d> | <lvl>{message}</lvl>",
                "colorize": True
            }
        ],
        "extra": {
            "container": name,
            "docker_timestamp": arrow.get(line[:22]).to('local')
        }
    }
    logger.configure(**config)
    if "ERROR:" in line or "FATAL:" in line or "CRITICAL:" in line:
        logger.error(line[31:])
    elif "WARNING:" in line:
        logger.warning(line[31:])
    elif "DEBUG:" in line:
        logger.debug(line[31:])
    else:
        logger.info(line[31:])


@click.command()
@click.argument('services', nargs=-1, required=True)
@click.option('--regex')
@click.pass_obj
def logs(meg: Megalus, services: list, regex: str):
    try:
        time_to_fetch = 2
        services_data_to_log = [
            meg.find_service(service_to_find)
            for service_to_find in services
        ]
        while True:
            for service_data in services_data_to_log:
                if len(services_data_to_log) > 1:
                    console.section(service_data['name'])
                containers = find_containers(service_data['name'])
                for container in containers:
                    log_data = container.logs(timestamps=True, stream=True, since=time_to_fetch)
                    for data in log_data:
                        if data:
                            line = data.decode('UTF-8').replace("\n", "")
                            if regex:
                                ret = re.search(regex, line)
                                if not ret:
                                    continue
                            show_log(container.name, line)
            sleep(time_to_fetch)
    except KeyboardInterrupt:
        sys.exit(0)



"""
Usage: geru bash <service>
This command will access bash inside the select service.
You can use this command on stopped containers.
Do not use this command to run unittests inside container.
To run unit tests please check the command geru test

#### Database migration:

For database migrate operations, first enter inside container, using the
command `geru bash <service>` and inside the container, type the needed
alembic command, replacing `alembic-development.ini` for
`alembic-docker.ini`. Make sure you have postgres running to enable the
migration `geru up postgres`

For example, to migrate Core database:

```bash
$ geru bash core
container@root$ alembic -c alembic-docker.ini upgrade head
```
"""

import click
from buzio import console
from loguru import logger

from megalus.main import Megalus
from megalus.utils import client, find_containers


@click.command()
@click.argument('service', nargs=1, required=True)
@click.pass_obj
def bash(meg: Megalus, service: str) -> None:
    service_data = meg.find_service(service)

    container_id = None
    eligible_containers = find_containers(service_data['name'])
    if not eligible_containers:
        logger.info("Running /bin/bash in service {}".format(service_data['name']))
        meg.run_command(
            'cd {} && docker-compose run --rm --service-ports {} /bin/bash'.format(service_data['working_dir'],
                                                                                   service_data['name'])
        )
    elif len(eligible_containers) == 1:
        container_id = eligible_containers[0].short_id
    else:
        container_names = [c.name for c in eligible_containers]
        container = console.choose(container_names, 'Please select the container')
        if container:
            container_id = client.containers.get(container).short_id
    if container_id:
        logger.info(
            "Running /bin/bash in service {} in container {}".format(service_data['name'], container_id))
        meg.run_command(
            'cd {} && docker exec -ti {} /bin/bash'.format(service_data['working_dir'], container_id)
        )
